/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2013-2015 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * http://glassfish.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.jersey.examples.entityfiltering.resource;

import java.lang.annotation.Annotation;
import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.Response;

import org.glassfish.jersey.examples.entityfiltering.domain.EntityStore;
import org.glassfish.jersey.examples.entityfiltering.domain.User;
import org.glassfish.jersey.examples.entityfiltering.filtering.UserDetailedView;

/**
 * Resource class for {@link User users}. Provides combined methods to retrieve users in "default" view and "detailed" view.
 * Detailed view can be triggered by appending {@code detailed} query param to the URI.
 * <p/>
 * To see the resource methods expanded refer to the {@link ProjectsResource}.
 *
 * @author Michal Gajdos
 * @see ProjectsResource
 */
@Path("users")
@Produces("application/json")
public class UsersResource {

    @GET
    public Response getUsers(@QueryParam("detailed") final boolean isDetailed) {
        return Response
                .ok()
                .entity(new GenericEntity<List<User>>(EntityStore.getUsers()) {},
                        isDetailed ? new Annotation[]{UserDetailedView.Factory.get()} : new Annotation[0])
                .build();
    }

    @GET
    @Path("{id}")
    public Response getUser(@PathParam("id") final Long id, @QueryParam("detailed") final boolean isDetailed) {
        return Response
                .ok()
                .entity(EntityStore.getUser(id),
                        isDetailed ? new Annotation[]{UserDetailedView.Factory.get()} : new Annotation[0])
                .build();
    }
}
